==> wreport_2.4.orig.tar.gz: wreport-2.4/wreport/uniconv.cc <==
/* C++ code produced by gperf version 3.0.3 */
/* Command-line: gperf  */
/* Computed positions: -k'1,4-7,12' */

#if !((' ' == 32) && ('!' == 33) && ('"' == 34) && ('#' == 35) \
      && ('%' == 37) && ('&' == 38) && ('\'' == 39) && ('(' == 40) \
      && (')' == 41) && ('*' == 42) && ('+' == 43) && (',' == 44) \
      && ('-' == 45) && ('.' == 46) && ('/' == 47) && ('0' == 48) \
      && ('1' == 49) && ('2' == 50) && ('3' == 51) && ('4' == 52) \
      && ('5' == 53) && ('6' == 54) && ('7' == 55) && ('8' == 56) \
      && ('9' == 57) && (':' == 58) && (';' == 59) && ('<' == 60) \
      && ('=' == 61) && ('>' == 62) && ('?' == 63) && ('A' == 65) \
      && ('B' == 66) && ('C' == 67) && ('D' == 68) && ('E' == 69) \
      && ('F' == 70) && ('G' == 71) && ('H' == 72) && ('I' == 73) \
      && ('J' == 74) && ('K' == 75) && ('L' == 76) && ('M' == 77) \
      && ('N' == 78) && ('O' == 79) && ('P' == 80) && ('Q' == 81) \
      && ('R' == 82) && ('S' == 83) && ('T' == 84) && ('U' == 85) \
      && ('V' == 86) && ('W' == 87) && ('X' == 88) && ('Y' == 89) \
      && ('Z' == 90) && ('[' == 91) && ('\\' == 92) && (']' == 93) \
      && ('^' == 94) && ('_' == 95) && ('a' == 97) && ('b' == 98) \
      && ('c' == 99) && ('d' == 100) && ('e' == 101) && ('f' == 102) \
      && ('g' == 103) && ('h' == 104) && ('i' == 105) && ('j' == 106) \
      && ('k' == 107) && ('l' == 108) && ('m' == 109) && ('n' == 110) \
      && ('o' == 111) && ('p' == 112) && ('q' == 113) && ('r' == 114) \
      && ('s' == 115) && ('t' == 116) && ('u' == 117) && ('v' == 118) \
      && ('w' == 119) && ('x' == 120) && ('y' == 121) && ('z' == 122) \
      && ('{' == 123) && ('|' == 124) && ('}' == 125) && ('~' == 126))
/* The character set is not based on ISO-646.  */
#error "gperf generated tables don't work with this execution character set. Please report a bug to <bug-gnu-gperf@gnu.org>."
#endif


#include <stdio.h>
#include <string.h>
#include "error.h"

namespace wreport {
struct uniconv { const char* tag; double mul; double add; };

#define TOTAL_KEYWORDS 92
#define MIN_WORD_LENGTH 4
#define MAX_WORD_LENGTH 24
#define MIN_HASH_VALUE 4
#define MAX_HASH_VALUE 142
/* maximum key range = 139, duplicates = 0 */

class Uniconv
{
private:
  static inline unsigned int hash (const char *str, unsigned int len);
public:
  static struct uniconv *find (const char *str, unsigned int len);
};

inline unsigned int
Uniconv::hash (register const char *str, register unsigned int len)
{
  static unsigned char asso_values[] =
    {
      143, 143, 143, 143, 143, 143, 143, 143, 143, 143,
      143, 143, 143, 143, 143, 143, 143, 143, 143, 143,
      143, 143, 143, 143, 143, 143, 143, 143, 143, 143,
      143, 143,  10,   5, 143, 143, 143,   0, 143, 143,
        5,  60,   0,  30, 143,  10,   0,   5,  15,   0,
       20,   5, 143, 143, 143, 143,   5, 143, 143, 143,
      143, 143,   0,  35, 143,  65,  10,   5,   5,   5,
       15,  15,   5, 143,  10,   0,  40,   5,   0,   5,
        0,  10,   5,  30,  70,   0,   5, 143, 143,  10,
      143, 143, 143, 143, 143, 143, 143,  20, 143,  10,
        0,   5,   0,   0,  60,  10,   5, 143, 143,   0,
      105,   0, 143, 143,   5,  40,   5, 143, 143, 143,
      143, 143, 143, 143, 143, 143, 143, 143, 143, 143,
      143, 143, 143, 143, 143, 143, 143, 143, 143, 143,
      143, 143, 143, 143, 143, 143, 143, 143, 143, 143,
      143, 143, 143, 143, 143, 143, 143, 143, 143, 143,
      143, 143, 143, 143, 143, 143, 143, 143, 143, 143,
      143, 143, 143, 143, 143, 143, 143, 143, 143, 143,
      143, 143, 143, 143, 143, 143, 143, 143, 143, 143,
      143, 143, 143, 143, 143, 143, 143, 143, 143, 143,
      143, 143, 143, 143, 143, 143, 143, 143, 143, 143,
      143, 143, 143, 143, 143, 143, 143, 143, 143, 143,
      143, 143, 143, 143, 143, 143, 143, 143, 143, 143,
      143, 143, 143, 143, 143, 143, 143, 143, 143, 143,
      143, 143, 143, 143, 143, 143, 143, 143, 143, 143,
      143, 143, 143, 143, 143, 143, 143
    };
  register int hval = len;

  switch (hval)
    {
      default:
        hval += asso_values[(unsigned char)str[11]];
      /*FALLTHROUGH*/
      case 11:
      case 10:
      case 9:
      case 8:
      case 7:
        hval += asso_values[(unsigned char)str[6]];
      /*FALLTHROUGH*/
      case 6:
        hval += asso_values[(unsigned char)str[5]];
      /*FALLTHROUGH*/
      case 5:
        hval += asso_values[(unsigned char)str[4]];
      /*FALLTHROUGH*/
      case 4:
        hval += asso_values[(unsigned char)str[3]+1];
      /*FALLTHROUGH*/
      case 3:
      case 2:
      case 1:
        hval += asso_values[(unsigned char)str[0]];
        break;
    }
  return hval;
}

struct uniconv *
Uniconv::find (register const char *str, register unsigned int len)
{
  static struct uniconv wordlist[] =
    {
      {""}, {""}, {""}, {""},
      {"m->M",				1,		0},
      {""},
      {"1/8->%",			12.5,		0},
      {""}, {""},
      {"K->C",				1,	-273.15001f},
      {"M->cm",			100,	0},
      {""}, {""}, {""}, {""},
      {"M->MM",			1000,	0},
      {""}, {""}, {""},
      {"MGP->m**2/s**2",		9.80665,	0},
      {"M->hm",			0.01,		0},
      {"M/S->m/s/10",			10,		0},
      {"C/10->K",			0.1,	273.15001f},
      {""},
      {"MGP->M**2/S**2",		9.80665,	0},
      {"KG/M**3->G/M**3",		1000,	0},
      {""},
      {"C/10->C",			0.1,	0},
      {"MGP->GPM",			1,		0},
      {"GPM->m**2/s**2",		9.80665,	0},
      {"m/sec->M/S",  			1,		0},
      {"KG/M**2->mm",			1,		0},
      {"K->C/10",			10,	-2731.5001f},
      {""},
      {"GPM->M**2/S**2",		9.80665,	0},
      {"%->PERCENT",			1,	0},
      {"%->1/8",			0.08,		0},
      {"C->C/10",			10,	0},
      {"MONTH->MONTHS",		1,	0},
      {"KG/M**2->KGM-2",		1,	0},
      {"G/G->KG/KG",			1,	0},
      {""}, {""},
      {"GPM->MGP",			1,		0},
      {"KGM-2->KG/M**2",		1,	0},
      {"mm->M",			0.001,		0},
      {""},
      {"Bq/L->BQ L-1",			1,		0},
      {"NBAR->PA",			10000,		0},
      {"C->K",				1,	273.15001f},
      {"MM->M",			0.001,	0},
      {"m/s/10->M/S",			0.1,		0},
      {""}, {""},
      {"SECOND->S",			1,	0},
      {"cm->M",			0.01,	0},
      {"S->sec",			1,	0},
      {""},
      {"mBar->PA",			100,		0},
      {"mm/10->KG/M**2",		0.1,		0},
      {"FT->M",			0.3048,	0},
      {"HOUR->HOURS",			1,	0},
      {"cal/cm**2->J/M**2",		41868,	0},
      {""},
      {"M**2/S**2->MGP",		0.101971621,	0},
      {"SECOND->SECONDS",		1,	0},
      {"mm->KG/M**2",			1,		0},
      {"JM-2->J/M**2",			1,	0},
      {"DEGREE**2->DEGREE2",		1,	0},
      {"m**2/s**2->MGP",		0.101971621,	0},
      {"DU->DOBSON",			1,		0},
      {"YEAR->YEARS",			1,	0},
      {"KPA->PA",			1000,	0},
      {"PA->mBar",			0.01,		0},
      {"M**2/S**2->GPM",		0.101971621,	0},
      {"KG/KG->G/G",			1,	0},
      {"sec->S",			1,	0},
      {"PA->Bar",			0.00001,	0},
      {"gsess->DEGREE TRUE",		1,		0},
      {"m**2/s**2->GPM",		0.101971621,	0},
      {"G/M**3->KG/M**3",		0.001,	0},
      {""},
      {"Bar->PA",			100000,		0},
      {"DEGREE TRUE->gsess",		1,		0},
      {"DAY->DAYS",			1,	0},
      {"SECONDS->SECOND",		1,	0},
      {"HOURS->HOUR",			1,	0},
      {"BQ L-1->Bq/L",			1,		0},
      {""},
      {"DEGREE TRUE->degree true",	1,		0},
      {"PERCENT->%",			1,	0},
      {"YEARS->YEAR",			1,	0},
      {"J/M**2->cal/cm**2",		0.000023885,	0},
      {""},
      {"degree true->DEGREE TRUE",	1,		0},
      {"M->FT",			3.2808,	0},
      {""}, {""},
      {"DEGREE2->DEGREE**2",		1,	0},
      {"DAYS->DAY",			1,	0},
      {"DOBSON->DU",			1,		0},
      {""},
      {"J/M**2->JM-2",			1,	0},
      {""}, {""},
      {"hm->M",			100,		0},
      {""},
      {"PA->KPA",			0.001,	0},
      {""},
      {"cal/s/cm**2->W/M**2",		41868,		0},
      {"MINUTE->MINUTES",		1,	0},
      {""},
      {"m**(2/3)/S->M**(2/3)/S",	1,	0},
      {""},
      {"M->m",				1,		0},
      {"M->mm",			1000,		0},
      {""},
      {"M**(2/3)/S->m**(2/3)/S",	1,	0},
      {"PA->NBAR",			0.0001,		0		},
      {"M/S->nodi",			1.94384,	0},
      {""}, {""},
      {"PA->hPa",			0.01,		0},
      {""},
      {"S->SECOND",			1,	0},
      {""}, {""},
      {"LOG (M-2)->LOG(1/M**2)",	1,		0},
      {""},
      {"cal/h/cm**2->W/M**2",		11.63,		0},
      {"MINUTES->MINUTE",		1,	0},
      {""},
      {"hPa->PA",			100,		0},
      {"MONTHS->MONTH",		1,	0},
      {"nodi->M/S",			0.51444,	0},
      {""}, {""}, {""}, {""}, {""}, {""}, {""},
      {"LOG(1/M**2)->LOG (M-2)",	1,		0}
    };

  if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
    {
      register int key = hash (str, len);

      if (key <= MAX_HASH_VALUE && key >= 0)
        {
          register const char *s = wordlist[key].tag;

          if (*str == *s && !strcmp (str + 1, s + 1))
            return &wordlist[key];
        }
    }
  return 0;
}


bool convert_units_allowed(const char* from, const char* to)
{
	if (strcmp(from, to) == 0
	    || strncmp(from, "CODE TABLE", 10) == 0
		|| strncmp(to, "CODE TABLE", 10) == 0
	    || strncmp(from, "FLAG TABLE", 10) == 0
		|| strncmp(to, "FLAG TABLE", 10) == 0)
	{
		return true;
	} else {
		char query[50];
		int len = snprintf(query, 48, "%s->%s", from, to);
		struct uniconv* info = Uniconv::find(query, len);
		return info != NULL;
	}
}

double convert_units(const char* from, const char* to, double val)
{
	if (strcmp(from, to) == 0
	    || strncmp(from, "CODE TABLE", 10) == 0
		|| strncmp(to, "CODE TABLE", 10) == 0
	    || strncmp(from, "FLAG TABLE", 10) == 0
		|| strncmp(to, "FLAG TABLE", 10) == 0)
		return val;

	char query[50];
	int len = snprintf(query, 48, "%s->%s", from, to);
	struct uniconv* info = Uniconv::find(query, len);
	if (info == NULL)
		error_notfound::throwf("looking for conversion from \"%s\" to \"%s\"", from, to);
	else
		return val * info->mul + info->add;
}

double convert_units_get_mul(const char* from, const char* to)
{
	if (strcmp(from, to) == 0
	    || strncmp(from, "CODE TABLE", 10) == 0
		|| strncmp(to, "CODE TABLE", 10) == 0
	    || strncmp(from, "FLAG TABLE", 10) == 0
		|| strncmp(to, "FLAG TABLE", 10) == 0)
	{
		return 1;
	} else {
		char query[50];
		int len = snprintf(query, 48, "%s->%s", from, to);
		struct uniconv* info = Uniconv::find(query, len);
		if (info == NULL)
			error_notfound::throwf("looking for conversion from \"%s\" to \"%s\"", from, to);
		else
			return info->mul;
	}
}

}
/* vim:set ts=4 sw=4: */

==> wreport_2.8.orig.tar.gz: wreport-2.8/wreport/uniconv.cc <==
/* C++ code produced by gperf version 3.0.3 */
/* Command-line: gperf  */
/* Computed positions: -k'1,4-5,11,$' */

#if !((' ' == 32) && ('!' == 33) && ('"' == 34) && ('#' == 35) \
      && ('%' == 37) && ('&' == 38) && ('\'' == 39) && ('(' == 40) \
      && (')' == 41) && ('*' == 42) && ('+' == 43) && (',' == 44) \
      && ('-' == 45) && ('.' == 46) && ('/' == 47) && ('0' == 48) \
      && ('1' == 49) && ('2' == 50) && ('3' == 51) && ('4' == 52) \
      && ('5' == 53) && ('6' == 54) && ('7' == 55) && ('8' == 56) \
      && ('9' == 57) && (':' == 58) && (';' == 59) && ('<' == 60) \
      && ('=' == 61) && ('>' == 62) && ('?' == 63) && ('A' == 65) \
      && ('B' == 66) && ('C' == 67) && ('D' == 68) && ('E' == 69) \
      && ('F' == 70) && ('G' == 71) && ('H' == 72) && ('I' == 73) \
      && ('J' == 74) && ('K' == 75) && ('L' == 76) && ('M' == 77) \
      && ('N' == 78) && ('O' == 79) && ('P' == 80) && ('Q' == 81) \
      && ('R' == 82) && ('S' == 83) && ('T' == 84) && ('U' == 85) \
      && ('V' == 86) && ('W' == 87) && ('X' == 88) && ('Y' == 89) \
      && ('Z' == 90) && ('[' == 91) && ('\\' == 92) && (']' == 93) \
      && ('^' == 94) && ('_' == 95) && ('a' == 97) && ('b' == 98) \
      && ('c' == 99) && ('d' == 100) && ('e' == 101) && ('f' == 102) \
      && ('g' == 103) && ('h' == 104) && ('i' == 105) && ('j' == 106) \
      && ('k' == 107) && ('l' == 108) && ('m' == 109) && ('n' == 110) \
      && ('o' == 111) && ('p' == 112) && ('q' == 113) && ('r' == 114) \
      && ('s' == 115) && ('t' == 116) && ('u' == 117) && ('v' == 118) \
      && ('w' == 119) && ('x' == 120) && ('y' == 121) && ('z' == 122) \
      && ('{' == 123) && ('|' == 124) && ('}' == 125) && ('~' == 126))
/* The character set is not based on ISO-646.  */
#error "gperf generated tables don't work with this execution character set. Please report a bug to <bug-gnu-gperf@gnu.org>."
#endif


#include <stdio.h>
#include <string.h>
#include "error.h"

namespace wreport {
struct uniconv { const char* tag; double mul; double add; };

#define TOTAL_KEYWORDS 102
#define MIN_WORD_LENGTH 4
#define MAX_WORD_LENGTH 24
#define MIN_HASH_VALUE 7
#define MAX_HASH_VALUE 186
/* maximum key range = 180, duplicates = 0 */

class Uniconv
{
private:
  static inline unsigned int hash (const char *str, unsigned int len);
public:
  static struct uniconv *find (const char *str, unsigned int len);
};

inline unsigned int
Uniconv::hash (register const char *str, register unsigned int len)
{
  static unsigned char asso_values[] =
    {
      187, 187, 187, 187, 187, 187, 187, 187, 187, 187,
      187, 187, 187, 187, 187, 187, 187, 187, 187, 187,
      187, 187, 187, 187, 187, 187, 187, 187, 187, 187,
      187, 187,   5, 187, 187, 187, 187,  40, 187, 187,
        0,   0,  60, 187, 187,   0, 187,  15,   0,   0,
        0,  20, 187, 187, 187, 187,   5, 187, 187, 187,
      187, 187,  15, 187, 187,   5,   5,   0,   5,   0,
       90,  10,   0,  30,  10,  20,  35,   5,  50,   0,
       40, 187,   0,  40,  15,  60, 187,  20, 187,  60,
      187, 187, 187, 187, 187, 187, 187,   0, 187,  45,
        0,  70, 187,  25,  70,   0, 187, 187,   0,   0,
       75, 187, 187, 187,   5,  25,  35,  35, 187, 187,
      187, 187, 187, 187, 187, 187, 187, 187, 187, 187,
      187, 187, 187, 187, 187, 187, 187, 187, 187, 187,
      187, 187, 187, 187, 187, 187, 187, 187, 187, 187,
      187, 187, 187, 187, 187, 187, 187, 187, 187, 187,
      187, 187, 187, 187, 187, 187, 187, 187, 187, 187,
      187, 187, 187, 187, 187, 187, 187, 187, 187, 187,
      187, 187, 187, 187, 187, 187, 187, 187, 187, 187,
      187, 187, 187, 187, 187, 187, 187, 187, 187, 187,
      187, 187, 187, 187, 187, 187, 187, 187, 187, 187,
      187, 187, 187, 187, 187, 187, 187, 187, 187, 187,
      187, 187, 187, 187, 187, 187, 187, 187, 187, 187,
      187, 187, 187, 187, 187, 187, 187, 187, 187, 187,
      187, 187, 187, 187, 187, 187, 187, 187, 187, 187,
      187, 187, 187, 187, 187, 187
    };
  register int hval = len;

  switch (hval)
    {
      default:
        hval += asso_values[(unsigned char)str[10]];
      /*FALLTHROUGH*/
      case 10:
      case 9:
      case 8:
      case 7:
      case 6:
      case 5:
        hval += asso_values[(unsigned char)str[4]];
      /*FALLTHROUGH*/
      case 4:
        hval += asso_values[(unsigned char)str[3]];
      /*FALLTHROUGH*/
      case 3:
      case 2:
      case 1:
        hval += asso_values[(unsigned char)str[0]];
        break;
    }
  return hval + asso_values[(unsigned char)str[len - 1]];
}

struct uniconv *
Uniconv::find (register const char *str, register unsigned int len)
{
  static struct uniconv wordlist[] =
    {
      {""}, {""}, {""}, {""}, {""}, {""}, {""},
      {"C/10->C",			0.1,	0},
      {""},
      {"M->m",				1,		0},
      {"M->mm",			1000,		0},
      {""}, {""}, {""},
      {"m->M",				1,		0},
      {""}, {""}, {""},
      {"mBar->PA",			100,		0},
      {"mm/10->KG/M**2",		0.1,		0},
      {""}, {""},
      {"C->C/10",			10,	0},
      {"DEGREE2->DEGREE**2",		1,	0},
      {"K->C",				1,	-273.15001f},
      {"M->MM",			1000,	0},
      {""},
      {"C/10->K",			0.1,	273.15001f},
      {""},
      {"M/S->nodi",			1.94384,	0},
      {"mm->M",			0.001,		0},
      {"M/S->m/s/10",			10,		0},
      {"Bar->PA",			100000,		0},
      {"MGP->GPM",			1,		0},
      {""},
      {"MM->M",			0.001,	0},
      {""}, {""},
      {"DEGREE**2->DEGREE2",		1,	0},
      {"KGM-2->KG/M**2",		1,	0},
      {"M->KM",			0.001,	0},
      {""},
      {"K->C/10",			10,	-2731.5001f},
      {""},
      {"C->K",				1,	273.15001f},
      {"G/G->KG/KG",			1,	0},
      {"mm->KG/M**2",			1,		0},
      {"KPA->PA",			1000,	0},
      {"DEGREE TRUE->gsess",		1,		0},
      {"m**2/s**2->GPM",		0.101971621,	0},
      {"KM->M",			1000,	0},
      {"HOURS->HOUR",			1,	0},
      {"Bq/L->BQ L-1",			1,		0},
      {"%->RATIO",				0.01,		0},
      {"M**2/S**2->GPM",		0.101971621,	0},
      {"M->cm",			100,	0},
      {""},
      {"LOG(1/M**2)->LOG (M-2)",	1,		0},
      {""},
      {"MGP->m**2/s**2",		9.80665,	0},
      {"ms/cm->S/M",			0.1,		0},
      {"1/8->%",			12.5,		0},
      {"m**(2/3)/S->M**(2/3)/S",	1,	0},
      {"NBAR->PA",			10000,		0},
      {"GPM->m**2/s**2",		9.80665,	0},
      {"S/M->ms/cm",			10,			0},
      {"%->1/8",			0.08,		0},
      {"M**(2/3)/S->m**(2/3)/S",	1,	0},
      {"PA->mBar",			0.01,		0},
      {"DAY->DAYS",			1,	0},
      {"KG/KG->G/G",			1,	0},
      {""},
      {"PA->Bar",			0.00001,	0},
      {"GPM->MGP",			1,		0},
      {"MGP->M**2/S**2",		9.80665,	0},
      {"cm->M",			0.01,	0},
      {""},
      {"LOG (M-2)->LOG(1/M**2)",	1,		0},
      {"RATIO->%",				100,		0},
      {"GPM->M**2/S**2",		9.80665,	0},
      {"M->hm",			0.01,		0},
      {""},
      {"JM-2->J/M**2",			1,	0},
      {"MONTHS->MONTH",		1,	0},
      {"m**2/s**2->MGP",		0.101971621,	0},
      {"DU->DOBSON",			1,		0},
      {"sec->S",			1,	0},
      {"PA->KPA",			0.001,	0},
      {"MONTH->MONTHS",		1,	0},
      {"M**2/S**2->MGP",		0.101971621,	0},
      {"PERCENT->%",			1,	0},
      {"HOUR->HOURS",			1,	0},
      {""},
      {"gsess->DEGREE TRUE",		1,		0},
      {"S->SECOND",			1,	0},
      {"Mj/m**2->J/M**2",		1000000,	0},
      {"KG/M**2->mm",			1,		0},
      {"hPa->PA",			100,		0},
      {""},
      {"DEGREE TRUE->degree true",	1,		0},
      {"hm->M",			100,		0},
      {""},
      {"BQ L-1->Bq/L",			1,		0},
      {""},
      {"cal/s/cm**2->W/M**2",		41868,		0},
      {"%->PERCENT",			1,	0},
      {"m/s/10->M/S",			0.1,		0},
      {""}, {""},
      {"KG/M**2->KGM-2",		1,	0},
      {"SECONDS->SECOND",		1,	0},
      {"YEARS->YEAR",			1,	0},
      {""},
      {"PA->NBAR",			0.0001,		0		},
      {"DAYS->DAY",			1,	0},
      {"DOBSON->DU",			1,		0},
      {""}, {""}, {""},
      {"minuti->S",                    60,     0},
      {"FT->M",			0.3048,	0},
      {""}, {""}, {""},
      {"nodi->M/S",			0.51444,	0},
      {"MINUTES->MINUTE",		1,	0},
      {""}, {""}, {""}, {""},
      {"M->FT",			3.2808,	0},
      {""},
      {"PA->hPa",			0.01,		0},
      {""}, {""},
      {"KG/M**3->G/M**3",		1000,	0},
      {""},
      {"cal/cm**2->J/M**2",		41868,	0},
      {""},
      {"SECOND->S",			1,	0},
      {""}, {""},
      {"J/M**2->JM-2",			1,	0},
      {""}, {""},
      {"SECOND->SECONDS",		1,	0},
      {""},
      {"J/M**2->cal/cm**2",		0.000023885,	0},
      {""},
      {"cal/h/cm**2->W/M**2",		11.63,		0},
      {""},
      {"YEAR->YEARS",			1,	0},
      {""}, {""},
      {"W/M**2->W/m**2",			1,		0},
      {""}, {""}, {""}, {""},
      {"W/m**2->W/M**2",			1,		0},
      {""}, {""}, {""}, {""}, {""},
      {"m/sec->M/S",  			1,		0},
      {""}, {""}, {""},
      {"degree true->DEGREE TRUE",	1,		0},
      {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""}, {""},
      {""},
      {"G/M**3->KG/M**3",		0.001,	0},
      {""}, {""}, {""}, {""},
      {"MINUTE->MINUTES",		1,	0},
      {"S->sec",			1,	0}
    };

  if (len <= MAX_WORD_LENGTH && len >= MIN_WORD_LENGTH)
    {
      register int key = hash (str, len);

      if (key <= MAX_HASH_VALUE && key >= 0)
        {
          register const char *s = wordlist[key].tag;

          if (*str == *s && !strcmp (str + 1, s + 1))
            return &wordlist[key];
        }
    }
  return 0;
}


bool convert_units_allowed(const char* from, const char* to)
{
	if (strcmp(from, to) == 0
	    || strncmp(from, "CODE TABLE", 10) == 0
		|| strncmp(to, "CODE TABLE", 10) == 0
	    || strncmp(from, "FLAG TABLE", 10) == 0
		|| strncmp(to, "FLAG TABLE", 10) == 0)
	{
		return true;
	} else {
		char query[50];
		int len = snprintf(query, 48, "%s->%s", from, to);
		struct uniconv* info = Uniconv::find(query, len);
		return info != NULL;
	}
}

double convert_units(const char* from, const char* to, double val)
{
	if (strcmp(from, to) == 0
	    || strncmp(from, "CODE TABLE", 10) == 0
		|| strncmp(to, "CODE TABLE", 10) == 0
	    || strncmp(from, "FLAG TABLE", 10) == 0
		|| strncmp(to, "FLAG TABLE", 10) == 0)
		return val;

	char query[50];
	int len = snprintf(query, 48, "%s->%s", from, to);
	struct uniconv* info = Uniconv::find(query, len);
	if (info == NULL)
		error_notfound::throwf("looking for conversion from \"%s\" to \"%s\"", from, to);
	else
		return val * info->mul + info->add;
}

double convert_units_get_mul(const char* from, const char* to)
{
	if (strcmp(from, to) == 0
	    || strncmp(from, "CODE TABLE", 10) == 0
		|| strncmp(to, "CODE TABLE", 10) == 0
	    || strncmp(from, "FLAG TABLE", 10) == 0
		|| strncmp(to, "FLAG TABLE", 10) == 0)
	{
		return 1;
	} else {
		char query[50];
		int len = snprintf(query, 48, "%s->%s", from, to);
		struct uniconv* info = Uniconv::find(query, len);
		if (info == NULL)
			error_notfound::throwf("looking for conversion from \"%s\" to \"%s\"", from, to);
		else
			return info->mul;
	}
}

}
/* vim:set ts=4 sw=4: */
